print("import: 'plotly'")
import plotly

print("import: 'plotly.api'")
import plotly.api

print("import: 'plotly.api.v1'")
import plotly.api.v1

print("import: 'plotly.api.v2'")
import plotly.api.v2

print("import: 'plotly.dashboard_objs'")
import plotly.dashboard_objs

print("import: 'plotly.graph_objs'")
import plotly.graph_objs

print("import: 'plotly.grid_objs'")
import plotly.grid_objs

print("import: 'plotly.offline'")
import plotly.offline

print("import: 'plotly.plotly'")
import plotly.plotly

print("import: 'plotly.plotly.chunked_requests'")
import plotly.plotly.chunked_requests

